root_dir="/tmp/thchs"

# Update package list
sudo apt update

# Install Python 3.8
sudo apt install python3-pip python3.8 python3.8-dev python3.8-distutils python3.8-venv -y

# Install rsync
sudo apt install rsync -y

# Reset "root_dir"
rm -rf "$root_dir"
mkdir -p "$root_dir"

# Create virtual environment
# All tools will be installed there and can only be run if the venv is activated
python3.8 -m venv "$root_dir/.venv"

# Activate environment
source "$root_dir/.venv/bin/activate"

# Install/update tools for installing software
python3.8 -m pip install \
  wheel==0.38.4 \
  setuptools==65.6.3

# Install tools
python3.8 -m pip install \
  textgrid-tools==0.0.7 \
  pronunciation-dictionary-utils==0.0.3 \
  speech-dataset-parser==0.0.3 \
  dict-from-dict==0.0.3 \
  dict-from-pypinyin==0.0.1

# Create conda environment
rm -rf "$root_dir/.venv-conda"
conda create -p "$root_dir/.venv-conda" -y

# Install Montreal Forced Aligner
conda install -y \
  -p "$root_dir/.venv-conda" \
  -c conda-forge montreal-forced-aligner==2.0.5

# Download THCHS-30 dataset from OpenSLR: https://www.openslr.org/18/
wget https://openslr.elda.org/resources/18/data_thchs30.tgz \
  -O "$root_dir/data_thchs30.tgz"
  
rm -rf "$root_dir/THCHS-30"
mkdir -p "$root_dir/THCHS-30"

# Unpack dataset
tar zxf "$root_dir/data_thchs30.tgz" \
  -C "$root_dir/THCHS-30"

# Convert dataset to a generic dataset to have all speakers separated in single folders
rm -rf "$root_dir/data"
dataset-converter-cli convert-thchs \
  "$root_dir/THCHS-30/data_thchs30" \
  "$root_dir/data" \
  --tier "Symbols" \
  --symlink --add-punctuation-marks

# Copy grids to new folder
rm -rf "$root_dir/grids"
rsync -am --include='*.TextGrid' -f 'hide,! */' $root_dir/data/ "$root_dir/grids"

# Merge syllables to words
textgrid-tools-cli intervals join-between-marks \
  "$root_dir/grids" -o \
  "Symbols" " " --join-with ""

# Export vocabulary
textgrid-tools-cli grids export-vocabulary \
  "$root_dir/grids" \
  "Symbols" \
  "$root_dir/vocabulary.txt" --ignore " "

# Merge words to sentences
textgrid-tools-cli intervals join \
  "$root_dir/grids" -o \
  "Symbols" --join-with ""

# Transcribe vocabulary to pinyin
dict-from-pypinyin-cli \
  "$root_dir/vocabulary.txt" \
  "$root_dir/dictionary-pinyin.dict" \
  --trim  "。" "？" \
  --style "TONE3"
  
dict-cli export-phonemes \
  "$root_dir/dictionary-pinyin.dict" \
  "$root_dir/dictionary-pinyin-phonemes.txt"

# Download pinyin to IPA mapping from https://zenodo.org/record/7525638
wget "https://zenodo.org/record/7525638/files/pinyin-ipa-map-TONE3-all.json" \
  -O "$root_dir/pinyin-to-ipa.json"

cp "$root_dir/dictionary-pinyin.dict" "$root_dir/dictionary-ipa.dict"

# Map pinyin in dictionary to IPA
dict-cli map-symbols-in-pronunciations-json \
  "$root_dir/dictionary-ipa.dict" \
  "$root_dir/pinyin-to-ipa.json"

# Select the first pronunciation because it is the most correct one
dict-cli select-single-pronunciation \
  "$root_dir/dictionary-ipa.dict" \
  --mode "first"

# Exporting phoneme-set to check that it is as expected
dict-cli export-phonemes \
  "$root_dir/dictionary-ipa.dict" \
  "$root_dir/dictionary-ipa-phoneme-set.txt"

# Create copy of dictionary which will be adjusted so that it can be taken for alignment with MFA
cp "$root_dir/dictionary-ipa.dict" "$root_dir/align.dict"

# Remove punctuation from pronunciations for alignment
dict-cli remove-symbols-from-pronunciations "$root_dir/align.dict" "。" "？"

# Create temporary copy to retrieve all phonemes to be able to see which phonemes need to be mapped so that MFA could align everything correctly
cp "$root_dir/align.dict" "$root_dir/align-tmp.dict"

# Remove tones
txt-utils-cli replace \
  "$root_dir/align-tmp.dict" \
  "[˩˨˧˦˥]+" ""

# Export phoneme-set
dict-cli export-phonemes \
  "$root_dir/align-tmp.dict" \
  "$root_dir/align-tmp-phonemes.dict"

# Remove temporary copy
rm -rf "$root_dir/align-tmp.dict"

# Download MFA dictionary to see which phonemes there exist
# see: https://mfa-models.readthedocs.io/en/latest/dictionary/Mandarin/Mandarin%20MFA%20dictionary%20v2_0_0a.html#mandarin-mfa-dictionary-v2-0-0a
wget https://github.com/MontrealCorpusTools/mfa-models/releases/download/dictionary-mandarin_mfa-v2.0.0a/mandarin_mfa.dict \
  -O "$root_dir/mandarin_mfa.dict"

cp "$root_dir/mandarin_mfa.dict" "$root_dir/mandarin_mfa-tmp.dict"

txt-utils-cli replace \
  "$root_dir/mandarin_mfa-tmp.dict" \
  "([0-9\.]+\t){4,}" ""

txt-utils-cli replace \
  "$root_dir/mandarin_mfa-tmp.dict" \
  "[˩˨˧˦˥]+" ""

dict-cli export-phonemes \
  "$root_dir/mandarin_mfa-tmp.dict" \
  "$root_dir/mandarin_mfa-tmp-phonemes.txt"

# Now compare "$root_dir/align-tmp-phonemes.dict" and "$root_dir/mandarin_mfa-tmp-phonemes.txt"
echo "$root_dir/align-tmp-phonemes.dict"
echo "$root_dir/mandarin_mfa-tmp-phonemes.txt"

# Change phonemes so that MFA could align the audio by mapping missing phonemes to the closest phoneme in MFA
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ai̯" "aj" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "au̯" "aw" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ei̯" "ej" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ou̯" "ow" -pm # toned
# ɻ exists but ʐ is presumable more close since ɹ̩ and ɻ̩ are z's, too
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɻ" "ʐ" -pm # untoned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɹ̩" "z̩" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɻ̩" "ʐ̩" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɚ" "ʐ̩" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɤ" "o" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ɛ" "e" -pm # toned
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ʊ" "u" -pm # toned

# Map tones:
# Tones in align: ˥, ˥˩, ˧˥, ˧˩˧, None
# Tones in MFA: ˥˥, ˥˩, ˦, ˧˥, ˨, ˨˩˦, ˩
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "˧˩˧" "~4" -pm # ˧˩˧ -> ˨˩˦
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "˧˥" "~3" -pm # ˧˥ -> ˧˥
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "˥˩" "~2" -pm # ˥˩ -> ˥˩ (required, otherwise the first character will be replaced in the next step)
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "˥" "~1" -pm # ˥ -> ˥˥
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "~1" "˥˥" -pm
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "~2" "˥˩" -pm
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "~3" "˧˥" -pm
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "~4" "˨˩˦" -pm

# Without mapping the toneless phonemes z̩ and ʐ̩ the alignment will contain "spn" which is problematic, therefore these are mapped to tone ˨ by choice
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "z̩" "z̩˨"
dict-cli map-symbols-in-pronunciations "$root_dir/align.dict" "ʐ̩" "ʐ̩˨"

# Export phoneme set for manual check that everything is how it should be
# Should be 82 phonemes
dict-cli export-phonemes \
  "$root_dir/align.dict" \
  "$root_dir/align-phoneme-set.txt"

# Change formatting to use TAB instead of space for MFA
dict-cli change-formatting \
  "$root_dir/align.dict" \
  -ps TAB

# Create config for alignment
# "♪" is chosen to ignore brackets
cat > "$root_dir/align.yaml" << EOF
beam: 240
retry_beam: 480
transition_scale: 1.0
acoustic_scale: 0.1
self_loop_scale: 0.1
boost_silence: 1.0
optional_silence_phone: sil
oov_phone: spn
ignore_case: False
punctuation:
clitic_markers:
compound_markers:
quote_markers:
word_break_markers:
brackets: [[♪, ♪]]
EOF

# Activate conda environment
deactivate
conda activate "$root_dir/.venv-conda"

# Download acoustic model for alignment (v2.0.0a)
# Note: TypeError is expected, but download works anyway
# https://mfa-models.readthedocs.io/en/latest/acoustic/Mandarin/Mandarin%20MFA%20acoustic%20model%20v2_0_0a.html
# https://github.com/MontrealCorpusTools/mfa-models/raw/main/acoustic/mandarin.zip
mfa model download acoustic mandarin_mfa

# Clear old alignments
rm -rf "$root_dir/grids_aligned"
mkdir -p "$root_dir/grids_aligned"

# Clear cache
rm -rf "$root_dir/MFA_tmp"
mkdir -p "$root_dir/MFA_tmp"

# Align each speaker separately in an own process (takes ~20min total)
# Re-execute the command until all speakers are transcribed (already transcribed will be skipped) because MFA may fail to align all on first round
find "$root_dir/grids" -mindepth 1 -type d -printf '%P\n' | xargs --max-procs 16 -I % \
  bash -c "if [[ ! -d '$root_dir/grids_aligned/%' ]]; then \
    mfa align \
      '$root_dir/grids/%' \
      '$root_dir/align.dict' \
      'mandarin_mfa' \
      '$root_dir/grids_aligned/%' \
      --config_path '$root_dir/align.yaml' \
      --audio_directory '$root_dir/data/%' \
      --temp_directory '$root_dir/MFA_tmp/%' \
      --num_jobs 1 \
      --output_format 'long_textgrid' \
      --verbose \
      --clean \
      --debug \
      --overwrite; \
  fi"

# Deactiavte aligner
conda deactivate

# Create backup in case something go wrong to not align again
rm -rf "$root_dir/grids_aligned_backup"
cp -r "$root_dir/grids_aligned" "$root_dir/grids_aligned_backup"

# Activate environment
source "$root_dir/.venv/bin/activate"

# Rename tier "phones" to "Phonemes"
textgrid-tools-cli tier rename \
  "$root_dir/grids_aligned" -o \
  "phones" \
  "Phonemes"

# Next step
rm -rf "$root_dir/grids_aligned_text_ipa"
cp -r "$root_dir/grids_aligned" "$root_dir/grids_aligned_text_ipa"

textgrid-tools-cli tier clone \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words" "words-tmp"

# Join all intervals into one interval containing the complete text with space between each word (through alignment space between words are only if there is a pause)
textgrid-tools-cli intervals join \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words-tmp" --join-with " "

# Separate each word by space and keep the space as separate interval between the words intervals (the space is needed for the next steps)
textgrid-tools-cli intervals split \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words-tmp" " " --keep

# Remove space-interval content
textgrid-tools-cli intervals replace-text \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words-tmp" \
  "^ $" ""

textgrid-tools-cli tier clone \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words-tmp" \
  "words-tmp-raw"

# Create copy of dictionary that will contain no punctuation
cp "$root_dir/dictionary-ipa.dict" "$root_dir/dictionary-ipa-raw.dict"

# Remove punctuation
dict-cli remove-symbols-from-pronunciations \
  "$root_dir/dictionary-ipa-raw.dict" \
  "。" "？"

dict-cli export-phonemes \
  "$root_dir/dictionary-ipa-raw.dict" \
  "$root_dir/dictionary-raw-phonemeset.txt"

# Transcribe words to IPA
textgrid-tools-cli tiers transcribe \
  "$root_dir/grids_aligned_text_ipa" -o \
  "$root_dir/dictionary-ipa.dict" \
  "words-tmp" -s 500

# Transcribe words to IPA (without punctuation)
textgrid-tools-cli tiers transcribe \
  "$root_dir/grids_aligned_text_ipa" -o \
  "$root_dir/dictionary-ipa-raw.dict" \
  "words-tmp-raw" -s 500

# Merge punctuation to the preceding phoneme
textgrid-tools-cli intervals join-symbols \
  "$root_dir/grids_aligned_text_ipa" -o \
  "words-tmp" \
  --join-symbols "。" "？" \
  --mode "right" --join-with " " \
  --ignore-join-symbols ""

rm -rf "$root_dir/grids_aligned_text_ipa_punc"
cp -r "$root_dir/grids_aligned_text_ipa" "$root_dir/grids_aligned_text_ipa_punc"

textgrid-tools-cli tier clone \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "Phonemes" \
  "phonemes-raw"
  
# Map correct phonemes to the aligned phonemes and overwrite the wrong phonemes used for alignment
textgrid-tools-cli tier map \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "words-tmp" \
  "Phonemes"

# Do same for phonemes containing no punctuation
textgrid-tools-cli tier map \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "words-tmp-raw" \
  "phonemes-raw"

# Remove tiers
textgrid-tools-cli tiers remove \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "words-tmp" "words-tmp-raw"

# Plot durations
textgrid-tools-cli grids plot-durations \
  "$root_dir/grids_aligned_text_ipa_punc" \
  "phonemes-raw" \
  "$root_dir/grids_aligned_text_ipa_punc.pdf"

# Plot durations for phonemes with tones ignored
textgrid-tools-cli tier clone \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "phonemes-raw" \
  "phonemes-raw-no-tones"

# Remove tone markers
textgrid-tools-cli intervals replace-text \
  "$root_dir/grids_aligned_text_ipa_punc" -o \
  "phonemes-raw-no-tones" \
  "˩|˧|˥" ""
  
# Plot durations
textgrid-tools-cli grids plot-durations \
  "$root_dir/grids_aligned_text_ipa_punc" \
  "phonemes-raw-no-tones" \
  "$root_dir/grids_aligned_text_ipa_punc-no-tones.pdf"

# Remove tier
textgrid-tools-cli tiers remove \
  "$root_dir/finalize" -o \
  "phonemes-raw-no-tones"
  
rm -rf "$root_dir/grids_mark_durations"
cp -r "$root_dir/grids_aligned_text_ipa_punc" "$root_dir/grids_mark_durations"

textgrid-tools-cli tier clone \
  "$root_dir/grids_mark_durations" -o \
  "phonemes-raw" \
  "durations" --ignore-marks
  
# Export phoneme-set to know which phonemes should be marked
textgrid-tools-cli grids export-vocabulary \
  "$root_dir/grids_mark_durations" \
  "phonemes-raw" \
  "$root_dir/grids_mark_durations/phoneme-set.txt"

# Mark short phonemes
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_durations" -o \
  "phonemes-raw" \
  "durations" \
  "˘" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 0 \
  --range-max 20 \
  --min-count 10 \
  --selection "a" "ai̯˥" "ai̯˥˩" "ai̯˧˥" "ai̯˧˩˧" "au̯˥" "au̯˥˩" "au̯˧˥" "au̯˧˩˧" "a˥" "a˥˩" "a˧˥" "a˧˩˧" "ei̯˥" "ei̯˥˩" "ei̯˧˥" "ei̯˧˩˧" "e˥" "e˥˩" "e˧˥" "e˧˩˧" "f" "i˥" "i˥˩" "i˧˥" "i˧˩˧" "j" "k" "kʰ" "l" "m" "n" "o" "ou̯˥" "ou̯˥˩" "ou̯˧˥" "ou̯˧˩˧" "o˥" "o˥˩" "o˧˥" "o˧˩˧" "p" "pʰ" "s" "t" "ts" "tsʰ" "tɕ" "tɕʰ" "tʰ" "u˥" "u˥˩" "u˧˥" "u˧˩˧" "w" "x" "y˥" "y˥˩" "y˧˥" "y˧˩˧" "ŋ" "ɕ" "ə" "ə˥" "ə˥˩" "ə˧˥" "ə˧˩˧" "ɚ˥˩" "ɚ˧˥" "ɚ˧˩˧" "ɛ˥" "ɛ˥˩" "ɛ˧˥" "ɛ˧˩˧" "ɤ" "ɤ˥" "ɤ˥˩" "ɤ˧˥" "ɤ˧˩˧" "ɥ" "ɹ̩" "ɹ̩˥" "ɹ̩˥˩" "ɹ̩˧˥" "ɹ̩˧˩˧" "ɻ" "ɻ̩" "ɻ̩˥" "ɻ̩˥˩" "ɻ̩˧˥" "ɻ̩˧˩˧" "ʂ" "ʈʂ" "ʈʂʰ" "ʊ˥" "ʊ˥˩" "ʊ˧˥" "ʊ˧˩˧"

# Normal phonemes aren't marked specifically

# Mark long phonemes
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_durations" -o \
  "phonemes-raw" \
  "durations" \
  "ˑ" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 80 \
  --range-max 90 \
  --min-count 10 \
  --selection "a" "ai̯˥" "ai̯˥˩" "ai̯˧˥" "ai̯˧˩˧" "au̯˥" "au̯˥˩" "au̯˧˥" "au̯˧˩˧" "a˥" "a˥˩" "a˧˥" "a˧˩˧" "ei̯˥" "ei̯˥˩" "ei̯˧˥" "ei̯˧˩˧" "e˥" "e˥˩" "e˧˥" "e˧˩˧" "f" "i˥" "i˥˩" "i˧˥" "i˧˩˧" "j" "k" "kʰ" "l" "m" "n" "o" "ou̯˥" "ou̯˥˩" "ou̯˧˥" "ou̯˧˩˧" "o˥" "o˥˩" "o˧˥" "o˧˩˧" "p" "pʰ" "s" "t" "ts" "tsʰ" "tɕ" "tɕʰ" "tʰ" "u˥" "u˥˩" "u˧˥" "u˧˩˧" "w" "x" "y˥" "y˥˩" "y˧˥" "y˧˩˧" "ŋ" "ɕ" "ə" "ə˥" "ə˥˩" "ə˧˥" "ə˧˩˧" "ɚ˥˩" "ɚ˧˥" "ɚ˧˩˧" "ɛ˥" "ɛ˥˩" "ɛ˧˥" "ɛ˧˩˧" "ɤ" "ɤ˥" "ɤ˥˩" "ɤ˧˥" "ɤ˧˩˧" "ɥ" "ɹ̩" "ɹ̩˥" "ɹ̩˥˩" "ɹ̩˧˥" "ɹ̩˧˩˧" "ɻ" "ɻ̩" "ɻ̩˥" "ɻ̩˥˩" "ɻ̩˧˥" "ɻ̩˧˩˧" "ʂ" "ʈʂ" "ʈʂʰ" "ʊ˥" "ʊ˥˩" "ʊ˧˥" "ʊ˧˩˧"
  
# Mark extra long phonemes
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_durations" -o \
  "phonemes-raw" \
  "durations" \
  "ː" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 90 \
  --range-max inf \
  --min-count 10 \
  --selection "a" "ai̯˥" "ai̯˥˩" "ai̯˧˥" "ai̯˧˩˧" "au̯˥" "au̯˥˩" "au̯˧˥" "au̯˧˩˧" "a˥" "a˥˩" "a˧˥" "a˧˩˧" "ei̯˥" "ei̯˥˩" "ei̯˧˥" "ei̯˧˩˧" "e˥" "e˥˩" "e˧˥" "e˧˩˧" "f" "i˥" "i˥˩" "i˧˥" "i˧˩˧" "j" "k" "kʰ" "l" "m" "n" "o" "ou̯˥" "ou̯˥˩" "ou̯˧˥" "ou̯˧˩˧" "o˥" "o˥˩" "o˧˥" "o˧˩˧" "p" "pʰ" "s" "t" "ts" "tsʰ" "tɕ" "tɕʰ" "tʰ" "u˥" "u˥˩" "u˧˥" "u˧˩˧" "w" "x" "y˥" "y˥˩" "y˧˥" "y˧˩˧" "ŋ" "ɕ" "ə" "ə˥" "ə˥˩" "ə˧˥" "ə˧˩˧" "ɚ˥˩" "ɚ˧˥" "ɚ˧˩˧" "ɛ˥" "ɛ˥˩" "ɛ˧˥" "ɛ˧˩˧" "ɤ" "ɤ˥" "ɤ˥˩" "ɤ˧˥" "ɤ˧˩˧" "ɥ" "ɹ̩" "ɹ̩˥" "ɹ̩˥˩" "ɹ̩˧˥" "ɹ̩˧˩˧" "ɻ" "ɻ̩" "ɻ̩˥" "ɻ̩˥˩" "ɻ̩˧˥" "ɻ̩˧˩˧" "ʂ" "ʈʂ" "ʈʂʰ" "ʊ˥" "ʊ˥˩" "ʊ˧˥" "ʊ˧˩˧"

# Next step
rm -rf "$root_dir/grids_mark_pauses"
cp -r "$root_dir/grids_mark_durations" "$root_dir/grids_mark_pauses"

# Mark pauses at begin and end (i.e. "both") as "SILX"
textgrid-tools-cli intervals replace-text \
  "$root_dir/grids_mark_pauses" -o \
  "phonemes-raw" \
  "^$" "SILX" --mode "both"

# Mark short breaks
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_pauses" -o \
  "phonemes-raw" \
  "durations" \
  "SIL1" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 0 \
  --range-max 33.33333333 \
  --min-count 3 \
  --selection "" 

# Mark normal pauses
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_pauses" -o \
  "phonemes-raw" \
  "durations" \
  "SIL2" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 33.33333333 \
  --range-max 66.66666666 \
  --min-count 3 \
  --selection "" 

# Mark long pauses
textgrid-tools-cli grids mark-durations \
  "$root_dir/grids_mark_pauses" -o \
  "phonemes-raw" \
  "durations" \
  "SIL3" \
  --marks-mode "separate" \
  --range-mode "percentile" \
  --scope "folder" \
  --range-min 66.66666666 \
  --range-max inf \
  --min-count 3 \
  --selection ""

# Next step: adding SIL0
rm -rf "$root_dir/add_SIL0"
cp -r "$root_dir/grids_mark_pauses" "$root_dir/add_SIL0"

textgrid-tools-cli tier clone \
  "$root_dir/add_SIL0" -o \
  "Phonemes" "Phonemes-tmp"
  
textgrid-tools-cli intervals join-by-boundary \
  "$root_dir/add_SIL0" -o \
  "words" "Phonemes-tmp" --join-with "|"
  
textgrid-tools-cli intervals join-between-marks \
  "$root_dir/add_SIL0" -o \
  "Phonemes-tmp" "" --join-with "|SIL0|"

# Split SIL0
textgrid-tools-cli intervals split \
  "$root_dir/add_SIL0" -o \
  "Phonemes-tmp" "|"

# Split punctuation
textgrid-tools-cli intervals split \
  "$root_dir/add_SIL0" -o \
  "Phonemes-tmp" " "

rm -rf "$root_dir/finalize"
cp -r "$root_dir/add_SIL0" "$root_dir/finalize"

# Merge phonemes with duration markers
# SILX must not be ignored/filtered because on tier "durations" the corresponding mark is ""
textgrid-tools-cli tier map \
  "$root_dir/finalize" -o \
  "durations" \
  "phonemes-raw" \
  --filter-from --filter-to --mode "append"

textgrid-tools-cli tier rename \
  "$root_dir/finalize" -o \
  "phonemes-raw" "phonemes"

textgrid-tools-cli tiers remove \
  "$root_dir/finalize" -o \
  "durations"
  
# Merge phonemes with duration markers but ignore punctuation and SIL0 in tier "Phonemes-tmp"
# This merges also SILX with "" correctly
textgrid-tools-cli tier map \
  "$root_dir/finalize" -o \
  "phonemes" \
  "Phonemes-tmp" --mode "replace" \
  --filter-to  "。" "？" "SIL0"

textgrid-tools-cli tier rename \
  "$root_dir/finalize" -o \
  "Phonemes-tmp" "transcription"

textgrid-tools-cli tier rename \
  "$root_dir/finalize" -o \
  "Phonemes" "dictionary"

textgrid-tools-cli intervals split \
  "$root_dir/finalize" -o \
  "dictionary" " "
  
textgrid-tools-cli tier map \
  "$root_dir/finalize" -o \
  "phonemes" \
  "dictionary" --mode "replace" \
  --filter-from "SILX" "SIL1" "SIL2" "SIL3" "" \
  --filter-to  "。" "？" ""

# Create narrow pronunciation dictionaries for each speaker
find "$root_dir/finalize" -mindepth 1 -type d -printf '%P\n' | xargs --max-procs 16 -I % \
  textgrid-tools-cli grids create-dictionary \
    "$root_dir/finalize/%" \
    "words" "dictionary" "pronunciations-narrow.dict" --scope "all" \
    --log "$root_dir/logs/%/log.txt"
  
# Create narrow pronunciation dictionary for all speakers
textgrid-tools-cli grids create-dictionary \
  "$root_dir/finalize" \
  "words" "dictionary" "pronunciations-narrow.dict" --scope "all"

# Remove tier
textgrid-tools-cli tiers remove \
  "$root_dir/finalize" -o \
  "dictionary"
  
textgrid-tools-cli tier move \
  "$root_dir/finalize" -o \
  "words" 1
  
textgrid-tools-cli tier move \
  "$root_dir/finalize" -o \
  "phonemes" 2
  
textgrid-tools-cli tier move \
  "$root_dir/finalize" -o \
  "transcription" 3
  
rm -rf "$root_dir/export"
cp -r "$root_dir/finalize" "$root_dir/export"

# Copy wav files
rsync -am --include='*.wav' -f 'hide,! */' --links $root_dir/data/ "$root_dir/export"

# Export words vocabulary
textgrid-tools-cli grids export-vocabulary \
  "$root_dir/export" \
  "words" \
  "$root_dir/words-vocabulary.txt"

# Export phonemes vocabulary
textgrid-tools-cli grids export-vocabulary \
  "$root_dir/export" \
  "phonemes" \
  "$root_dir/phonemes-vocabulary.txt"

# Export transcription vocabulary
textgrid-tools-cli grids export-vocabulary \
  "$root_dir/export" \
  "transcription" \
  "$root_dir/transcription-vocabulary.txt"

# Plot durations
textgrid-tools-cli grids plot-durations \
  "$root_dir/export" \
  "phonemes" \
  "$root_dir/phoneme-durations-extended.pdf"

# Create statistics
# Max: ca. 13.85s, Avg: 7.64s
# Total duration: 122972.59s / 2049.54min / 34.16h
textgrid-tools-cli grids plot-stats \
  "$root_dir/export" -o \
  "$root_dir/stats.pdf" \
  "$root_dir/stats.csv"

# Copy data required for restoring original folder structure
cp "$root_dir/data/filename-mapping.json" "$root_dir/export/filename-mapping.json"

# Restore original folder structure
rm -rf "$root_dir/THCHS-30-final"
dataset-converter-cli restore-structure \
  "$root_dir/export" \
  "$root_dir/THCHS-30-final" --symlink

# Prepare upload data
upload_dir="$root_dir/upload"

rm -rf "$upload_dir"
mkdir -p "$upload_dir"

cd "$root_dir/THCHS-30-final"
rm -rf "$upload_dir/grids.zip"
zip -q "$upload_dir/grids.zip" -r . -i '*.TextGrid'

cd "$root_dir/export"
rm -rf "$upload_dir/grids-sdp.zip"
zip -q "$upload_dir/grids-sdp.zip" -r . -i '*.TextGrid'

cp "$root_dir/finalize/pronunciations-narrow.dict" "$upload_dir/pronunciations-narrow.dict"
cp "$root_dir/words-vocabulary.txt" "$upload_dir/words-vocabulary.txt"
cp "$root_dir/phonemes-vocabulary.txt" "$upload_dir/phonemes-vocabulary.txt"
cp "$root_dir/transcription-vocabulary.txt" "$upload_dir/transcription-vocabulary.txt"
cp "$root_dir/phoneme-durations-extended.pdf" "$upload_dir/phonemes-durations.pdf"
cp "$root_dir/grids_aligned_text_ipa_punc.pdf" "$upload_dir/phonemes-durations-simple.pdf"
cp "$root_dir/grids_aligned_text_ipa_punc-no-tones.pdf" "$upload_dir/phonemes-durations-simple-toneless.pdf"
cp "$root_dir/dictionary-ipa.dict" "$upload_dir/pronunciations-broad.dict"

# Create speaker narrow dictionaries
rm -rf "$root_dir/speaker-dictionaries"
mkdir -p "$root_dir/speaker-dictionaries"
find "$root_dir/finalize" -mindepth 1 -type d -printf '%P\n' | xargs --max-procs 16 -I % \
  cp "$root_dir/finalize/%/pronunciations-narrow.dict" "$root_dir/speaker-dictionaries/pronunciations-narrow-%.dict"
cd "$root_dir/speaker-dictionaries"
rm -rf "$upload_dir/pronunciations-narrow-speakers.zip"
zip -q "$upload_dir/pronunciations-narrow-speakers.zip" -r .
